
HealPoints = {
	ENABLED = nil;
	lastLink = nil;
	diff1  = nil;
	diff2 = nil;
	diff3 = nil;
	diff4 = nil;
}

function HealPointsFrame_OnLoad()
	local _, className = UnitClass("player");
	if (className == "PALADIN" or className == "PRIEST" or className == "DRUID" or className == "SHAMAN") then
		HealPoints.ENABLED = 1;
	end

	if (HealPoints.ENABLED) then
		SlashCmdList["HEALPOINTSCOMMAND"] = HealPoints_SlashHandler;
		SLASH_HEALPOINTSCOMMAND1 = "/healpoints";

		-- Register events
		this:RegisterEvent("UNIT_AURA");
		this:RegisterEvent("UNIT_LEVEL");
		this:RegisterEvent("UNIT_MAXMANA");
		this:RegisterEvent("UNIT_STATS");
		this:RegisterEvent("LEARNED_SPELL_IN_TAB");
		this:RegisterEvent("CHARACTER_POINTS_CHANGED");
	
		local oldFunc = HealPointsBS_Update;
		HealPointsBS_Update = function () 
			oldFunc(); 
			HealPoints.lastLink = nil;
			if (CharacterFrame:IsVisible()) then
				HealPointsFrame_SetPoints();
			end
		end
		
		local oldSetBagItem = GameTooltip.SetBagItem;
		GameTooltip.SetBagItem = function(this, bag, slot)			
			local hasCooldown, repairCost = oldSetBagItem(this, bag, slot);
			local link = GetContainerItemLink(bag, slot);
			HealPoints:DrawTooltip(GameTooltip, link);
			return hasCooldown, repairCost;		
		end
		
		local oldSetLootItem = GameTooltip.SetLootItem;
		GameTooltip.SetLootItem = function(this, slot)
			oldSetLootItem(this, slot);
			local link = GetLootSlotLink(slot);
			HealPoints:DrawTooltip(GameTooltip, link);
		end

		local oldSetQuestItem = GameTooltip.SetQuestItem
		GameTooltip.SetQuestItem = function(this, unit, slot)
			oldSetQuestItem(this, unit, slot);
			local link = GetQuestItemLink(unit, slot);
			HealPoints:DrawTooltip(GameTooltip, link);			
		end
	
		local oldSetQuestLogItem = GameTooltip.SetQuestLogItem
		GameTooltip.SetQuestLogItem = function(this, sOpt, slot)
			oldSetQuestLogItem(this, sOpt, slot);
			local link = GetQuestLogItemLink(sOpt, slot);
			HealPoints:DrawTooltip(GameTooltip, link);			
		end
		
		local oldSetTradeSkillItem = GameTooltip.SetTradeSkillItem
		GameTooltip.SetTradeSkillItem = function(this, skill, slot)
			oldSetTradeSkillItem(this, skill, slot);
			local link = (slot) and GetTradeSkillReagentItemLink(skill, slot) or GetTradeSkillItemLink(skill);
			HealPoints:DrawTooltip(GameTooltip, link);			
		end

		local oldSetMerchantItem = GameTooltip.SetMerchantItem
		GameTooltip.SetMerchantItem = function(this, slot)
			oldSetMerchantItem(this, slot);
			local link = GetMerchantItemLink(slot);
			HealPoints:DrawTooltip(GameTooltip, link);			
		end

		local oldSetAuctionItem = GameTooltip.SetAuctionItem
		GameTooltip.SetAuctionItem = function(this, unit, slot)
			oldSetAuctionItem(this, unit, slot);
			local link = GetAuctionItemLink(unit, slot);
			HealPoints:DrawTooltip(GameTooltip, link);			
		end

		local oldSetInventoryItem = GameTooltip.SetInventoryItem
		GameTooltip.SetInventoryItem = function(this, unit, slot)
			local sItem, sCooldown, sRepair = oldSetInventoryItem(this, unit, slot);
			if(not sItem) then return nil; end
			if (slot > 39) then -- bank
				local link = GetInventoryItemLink(unit, slot);
				HealPoints:DrawTooltip(GameTooltip, link);
			end
			return sItem, sCooldown, sRepair;			
		end
		
		local oldSetItemRef = SetItemRef
		SetItemRef = function(link, name, button)
			oldSetItemRef(link, name, button);
			if(not link or not name) then return nil; end
			if(not ItemRefTooltip) then return nil; end
			if( strsub(link, 1, 6) ~= "Player" ) then	
				if( ItemRefTooltip:IsVisible() and link) then
					if (link) then
						if(not DressUpFrame:IsVisible()) then
							HealPoints:DrawTooltip(ItemRefTooltip, link);			
						end
						ItemRefTooltip.isDisplayDone = nil;
					end
				end
			end
		end
		
		DEFAULT_CHAT_FRAME:AddMessage("HealPoints loaded.",1,1,1);
	else
		DEFAULT_CHAT_FRAME:AddMessage("HealPoints not loaded - not a healing class",1,1,1);
	end
end

function HealPoints:DrawTooltip(frame, reference)
	local function colorValue(value)
		local v = format("%5.0f", value);
		if (value == 0) then
			return v;
		elseif (value > 0) then
			return GREEN_FONT_COLOR_CODE..v..FONT_COLOR_CODE_CLOSE;
		else
			return RED_FONT_COLOR_CODE..v..FONT_COLOR_CODE_CLOSE;
		end
	end

	local function addLine(text, diff1, diff2)
		if (diff1 == diff2) then
			frame:AddDoubleLine(text, colorValue(diff1), 		
			NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b,
			NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
		else
			frame:AddDoubleLine(text, colorValue(diff1).." / "..colorValue(diff2), 		
			NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b,
			NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);			
		end		
	end
	
	if (reference ~= nil and string.find(reference, "item", 0, true) ~= nil) then
		if (reference ~= HealPoints.lastLink) then
			HealPoints.diff1, HealPoints.diff2, HealPoints.diff3, HealPoints.diff4 = HealPointsCalculator_ComputeHealpointsDiff(frame, reference)
			HealPoints.lastLink = reference;
		end
		if (HealPoints.diff3 ~= nil and HealPoints.diff3 ~= 0 and HealPoints.diff4 ~= nil and HealPoints.diff4 ~= 0) then
			addLine("HealPoints difference, top slot:", HealPoints.diff1, HealPoints.diff2);
			addLine("HealPoints difference, bottom slot:", HealPoints.diff3, HealPoints.diff4);
			frame:Show();
		elseif (HealPoints.diff1 ~= 0 or HealPoints.diff2 ~= 0) then
			addLine("HealPoints difference:", HealPoints.diff1, HealPoints.diff2);
			frame:Show();
		end;
	end;
end

function HealPoints_SlashHandler(msg)
	if (msg == 'bscan') then
		local _, className = UnitClass("player");
		DEFAULT_CHAT_FRAME:AddMessage("Special set/equip bonuses detected by HealPoints:",1,1,1);		
		DEFAULT_CHAT_FRAME:AddMessage("==========================================",1,1,1);		
		DEFAULT_CHAT_FRAME:AddMessage("# % mana regeneration while casting: "..HealPointsBS:GetBonus('CASTINGREG'),1,1,1);		
		DEFAULT_CHAT_FRAME:AddMessage("# Increased Spell crit chance in %: "..HealPointsBS:GetBonus('SPELLCRIT'),1,1,1);		
		if (className == "PALADIN") then
			DEFAULT_CHAT_FRAME:AddMessage("# Reduced casting time Holy Light: 0."..HealPointsBS:GetBonus('CASTINGHOLYLIGHT'),1,1,1);		
			DEFAULT_CHAT_FRAME:AddMessage("# Additional +healing for Flash of Light: "..HealPointsBS:GetBonus('IMPFLASHOFLIGHT'),1,1,1);		
		elseif (className == "PRIEST") then
			DEFAULT_CHAT_FRAME:AddMessage("# Reduced casting time Flash Heal: 0."..HealPointsBS:GetBonus('CASTINGFLASHHEAL'),1,1,1);		
			DEFAULT_CHAT_FRAME:AddMessage("# Increased duration Renew: "..HealPointsBS:GetBonus('DURATIONRENEW'),1,1,1);		
			DEFAULT_CHAT_FRAME:AddMessage("# Increased Holy spell crit chance in %: "..HealPointsBS:GetBonus('HOLYCRIT'),1,1,1);		
		elseif (className == "DRUID") then
			DEFAULT_CHAT_FRAME:AddMessage("# Reduced casting time Regrowth: 0."..HealPointsBS:GetBonus('CASTINGREGROWTH'),1,1,1);		
			DEFAULT_CHAT_FRAME:AddMessage("# Reduced casting time Healing Touch: 0."..HealPointsBS:GetBonus('CASTINGHEALINGTOUCH'),1,1,1);
			DEFAULT_CHAT_FRAME:AddMessage("# Increased duration Rejuvenation: "..HealPointsBS:GetBonus('DURATIONREJUV'),1,1,1);		
			DEFAULT_CHAT_FRAME:AddMessage("# Increased Nature spell crit chance in %: "..HealPointsBS:GetBonus('NATURECRIT'),1,1,1);
			DEFAULT_CHAT_FRAME:AddMessage("# Additional +healing for Rejuvenation: "..HealPointsBS:GetBonus('IMPREJUVENATION'),1,1,1);
		elseif (className == "SHAMAN") then
			DEFAULT_CHAT_FRAME:AddMessage("# Reduced casting time Chain Heal: 0."..HealPointsBS:GetBonus('CASTINGCHAINHEAL'),1,1,1);		
			DEFAULT_CHAT_FRAME:AddMessage("# Increased Nature spell crit chance in %: "..HealPointsBS:GetBonus('NATURECRIT'),1,1,1);
			DEFAULT_CHAT_FRAME:AddMessage("# Additional +healing for Lesser Healing Wave: "..HealPointsBS:GetBonus('IMPLESSERHEALINGWAVE'),1,1,1);
			DEFAULT_CHAT_FRAME:AddMessage("# Increased amount healed by Chain Heal to targets beyond first in %: "..HealPointsBS:GetBonus('IMPCHAINHEAL'),1,1,1);		
			DEFAULT_CHAT_FRAME:AddMessage("# 25% chance when casting (Lesser) Healing Wave to get "..HealPointsBS:GetBonus('REFUNDHEALINGWAVE').. "% of the mana cost refunded",1,1,1);
			DEFAULT_CHAT_FRAME:AddMessage("# Healing Wave jumps to two targets with an effect reduction of : "..HealPointsBS:GetBonus('JUMPHEALINGWAVE').."% (0% = no jumping)",1,1,1);
		end
	elseif (msg == 'config') then
		if (HealPointsConfig:IsVisible()) then
			HealPointsConfig:Hide();
		else
			HealPointsConfig:Show();
		end
	else
		if(HealPointsCalculator:IsVisible()) then
			HealPointsCalculator:Hide();
		else
			HealPointsCalculator:Show();
		end
	end
end

function HealPointsFrame_OnEvent(event)
	if (not CharacterFrame:IsVisible() or HealPoints.ENABLED == nil) then
		return;
	end
	-- Do nothing if event target is not player
	if ((	event == "UNIT_AURA" or 
			event == "UNIT_LEVEL" or 
			event == "UNIT_MAXMANA" or 
			event == "UNIT_STATS") 
			and not UnitIsUnit(arg1, "player")) then
		return;
	end	
	HealPointsFrame_SetPoints();
end

function HealPointsFrame_OnEnter()
	if (HealPoints.ENABLED) then
		GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
		GameTooltip:SetText("HealPoints "..format("%5.0f", this.healPoints),
								HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		GameTooltip:AddDoubleLine("Mana regeneration while casting/5s:",
										format("%5.0f", this.manaRegenCasting),
										NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b,
										HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		GameTooltip:AddDoubleLine("Normal mana regeneration/5s:",
										format("%5.0f", this.manaRegenNormal),
										NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b,
										HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		GameTooltip:AddDoubleLine("+Healing:",
										format("%5.0f", this.healing),
										NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b,
										HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		GameTooltip:AddDoubleLine("Chance to crit with healing spells:",
										format("%.2f%%", this.spellCrit),
										NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b,
										HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		GameTooltip:AddLine("Hint: Click to show the HealPoints calculator",
								GREEN_FONT_COLOR.r, GREEN_FONT_COLOR.g, GREEN_FONT_COLOR.b);
		GameTooltip:Show();
	end
end

function HealPointsFrame_OnShow()
	if (HealPoints.ENABLED) then
		HealPointsFrame_SetPoints();
	end
end

function HealPointsFrame_OnClick()
	if (HealPointsCalculator:IsVisible()) then
		HealPointsCalculator:Hide();
	elseif (HealPoints.ENABLED) then
		HealPointsCalculator:Show();
	end
end

function HealPointsFrame_SetPoints()
	local label = getglobal("HealPointsFrameLabel");
	local text = getglobal("HealPointsFrameStatText");
	local frame = getglobal("HealPointsFrame");

	local manaRegenNormal, manaRegenCasting = HealPointsCalculator_ComputeManaRegen();
	local healing = HealPointsCalculator_ComputeHealing();
	local spellCrit = HealPointsCalculator_ComputeSpellCrit();
	local mana = UnitManaMax("player");
	local healPoints = HealPointsCalculator_ComputeHealpoints(healing, spellCrit, mana, manaRegenCasting, manaRegenNormal);
	
	frame.healPoints = healPoints;
	frame.manaRegenCasting = manaRegenCasting;
	frame.manaRegenNormal = manaRegenNormal;
	frame.healing = healing;
	frame.spellCrit = spellCrit;

	label:SetText("HealPoints:");
	text:SetText(format("%5.0f", healPoints));
end
