
HEALPOINTS_CONFIG = { };
local valueUpdating = 0;

local SPELLLIST = { };

local Paladin_Spell = { HPC_SPELL_FOL, HPC_SPELL_HL };
local Priest_Spell = { HPC_SPELL_FH, HPC_SPELL_LH, HPC_SPELL_HEAL, HPC_SPELL_GH };
local Druid_Spell = { HPC_SPELL_HT };
local Shaman_Spell = { HPC_SPELL_LHW, HPC_SPELL_HW, HPC_SPELL_CHAIN };

local function getHighestSpellRank(spellName)
	local totalSpellTabs = GetNumSpellTabs();
	local totalSpells = 0;
	for t = 1, totalSpellTabs do
		local _, _, _, numSpells = GetSpellTabInfo(t);
		totalSpells = totalSpells + numSpells;
	end

	local highestRank = 0;
	for t = 1, totalSpells do
		local nameSpell, rankSpell = GetSpellName(t, BOOKTYPE_SPELL);	
		if (nameSpell == spellName) then		
			local j, k = string.find(rankSpell, HPC_DIV_RANK.." ");	
			if (j ~= nil) then
				highestRank = math.max(highestRank, string.sub(rankSpell, k));
			end
		end
	end
	return highestRank;
end

local function getTableIndex(t, entry)
	for i = 1, table.getn(t), 1 do
		if (t[i] == entry) then
			return i;
		end
	end
end

local function updateSpellLists()
	local function updateSpellList(objref, funct)
		for i = 1, table.getn(SPELLLIST), 1 do
			local info = {};
			info.text = SPELLLIST[i];
			info.func = funct;
			if (objref and type(objref) == "table") then
				info.owner = objref;
			end
			UIDropDownMenu_AddButton(info)
		end
	end

	local _, className = UnitClass("player");
	SPELLLIST = { };
	if (className == "PALADIN") then
		for i = 1, table.getn(Paladin_Spell), 1 do 
			if (getHighestSpellRank(Paladin_Spell[i]) > 0) then
				table.insert(SPELLLIST, Paladin_Spell[i]);
			end
		end
	elseif (className == "PRIEST") then
		for i = 1, table.getn(Priest_Spell), 1 do 
			if (getHighestSpellRank(Priest_Spell[i]) > 0) then
				table.insert(SPELLLIST, Priest_Spell[i]);
			end
		end
	elseif (className == "DRUID") then
		for i = 1, table.getn(Druid_Spell), 1 do 
			if (getHighestSpellRank(Druid_Spell[i]) > 0) then
				table.insert(SPELLLIST, Druid_Spell[i]);
			end
		end
	elseif (className == "SHAMAN") then
		for i = 1, table.getn(Shaman_Spell), 1 do 
			if (getHighestSpellRank(Shaman_Spell[i]) > 0) then
				table.insert(SPELLLIST, Shaman_Spell[i]);
			end
		end
	end

	if (HEALPOINTS_CONFIG['power']['spell'] == '') then
		HEALPOINTS_CONFIG['power']['spell'] = SPELLLIST[1];
	end
	if (HEALPOINTS_CONFIG['endurance']['spell'] == '') then
		HEALPOINTS_CONFIG['endurance']['spell'] = SPELLLIST[1];
	end
	
	UIDropDownMenu_Initialize(HealPointsPowerConfigManualSelectSpell,function() updateSpellList(HealPointsPowerConfigManualSelectSpell, HealPointsConfigPowerSpellSelect_OnClick); end);
	UIDropDownMenu_Initialize(HealPointsEnduranceConfigManualSelectSpell,function() updateSpellList(HealPointsEnduranceConfigManualSelectSpell, HealPointsConfigEnduranceSpellSelect_OnClick); end);
end

local function updateRankLists()
	local function updateRankList(objref, maxRank)
		for i = 1, maxRank, 1 do
			local info = {};
			info.text = "Rank "..i;
			info.func = HealPointsConfigRankSelect_OnClick;
			if (objref and type(objref) == "table") then
				info.owner = objref;
			end
			UIDropDownMenu_AddButton(info)
		end
	end

	local powerMax = getHighestSpellRank(HEALPOINTS_CONFIG['power']['spell']);
	UIDropDownMenu_Initialize(HealPointsPowerConfigManualSelectRank,function() updateRankList(HealPointsPowerConfigManualSelectRank, powerMax); end);

	local enduranceMax = getHighestSpellRank(HEALPOINTS_CONFIG['endurance']['spell']);
	UIDropDownMenu_Initialize(HealPointsEnduranceConfigManualSelectRank,function() updateRankList(HealPointsEnduranceConfigManualSelectRank, enduranceMax); end);
end

local function updateGUI() -- Updates GUI to match HEALPOINTS_CONFIG
	valueUpdating = 1;
	-- PowerPoints
	getglobal("HealPointsPowerConfigDuration"):SetValue(HEALPOINTS_CONFIG['power']['duration']);
	getglobal("HealPointsPowerConfigDurationText"):SetText(HEALPOINTS_CONFIG['power']['duration'].." min");
	if (HEALPOINTS_CONFIG['power']['auto'] == 1) then
		getglobal("HealPointsPowerConfigAutoSelect"):SetChecked(1);
		getglobal("HealPointsPowerConfigManualSelect"):SetChecked(0);
	else
		getglobal("HealPointsPowerConfigAutoSelect"):SetChecked(0);
		getglobal("HealPointsPowerConfigManualSelect"):SetChecked(1);
	end
	
	-- EndurancePoints
	getglobal("HealPointsEnduranceConfigDuration"):SetValue(HEALPOINTS_CONFIG['endurance']['duration']);
	getglobal("HealPointsEnduranceConfigDurationText"):SetText(HEALPOINTS_CONFIG['endurance']['duration'].." min");
	if (HEALPOINTS_CONFIG['endurance']['auto'] == 1) then
		getglobal("HealPointsEnduranceConfigAutoSelect"):SetChecked(1);
		getglobal("HealPointsEnduranceConfigManualSelect"):SetChecked(0);
	else
		getglobal("HealPointsEnduranceConfigAutoSelect"):SetChecked(0);
		getglobal("HealPointsEnduranceConfigManualSelect"):SetChecked(1);
	end

	-- SpellLists
	if (UIDropDownMenu_GetSelectedID(HealPointsPowerConfigManualSelectSpell) ~= getTableIndex(SPELLLIST, HEALPOINTS_CONFIG['power']['spell'])) then
		UIDropDownMenu_SetSelectedID(HealPointsPowerConfigManualSelectSpell, getTableIndex(SPELLLIST, HEALPOINTS_CONFIG['power']['spell']));
	end
	if (UIDropDownMenu_GetSelectedID(HealPointsEnduranceConfigManualSelectSpell) ~= getTableIndex(SPELLLIST, HEALPOINTS_CONFIG['endurance']['spell'])) then
		UIDropDownMenu_SetSelectedID(HealPointsEnduranceConfigManualSelectSpell, getTableIndex(SPELLLIST, HEALPOINTS_CONFIG['endurance']['spell']));
	end
	updateRankLists();
	UIDropDownMenu_SetSelectedID(HealPointsPowerConfigManualSelectRank, HEALPOINTS_CONFIG['power']['rank']);
	UIDropDownMenu_SetSelectedID(HealPointsEnduranceConfigManualSelectRank, HEALPOINTS_CONFIG['endurance']['rank']);
	
	-- RegenPoints
	getglobal("HealPointsRegenConfigDuration"):SetValue(HEALPOINTS_CONFIG['regen']['duration']);
	getglobal("HealPointsRegenConfigDurationText"):SetText(HEALPOINTS_CONFIG['regen']['duration'].." min");
	
	valueUpdating = 0;
end

function HealPointsConfigPowerSpellSelect_OnClick()
	if (this.owner) then
		UIDropDownMenu_SetSelectedID(this.owner, this:GetID());
		HEALPOINTS_CONFIG['power']['spell'] = SPELLLIST[UIDropDownMenu_GetSelectedID(HealPointsPowerConfigManualSelectSpell)];
		updateRankLists();
		HEALPOINTS_CONFIG['power']['rank'] = 1;
		UIDropDownMenu_SetSelectedID(HealPointsPowerConfigManualSelectRank, 1);
		HealPointsCalculator_ConfigUpdated();
	end
end

function HealPointsConfigEnduranceSpellSelect_OnClick()
	if (this.owner) then
		UIDropDownMenu_SetSelectedID(this.owner, this:GetID());
		HEALPOINTS_CONFIG['endurance']['spell'] = SPELLLIST[UIDropDownMenu_GetSelectedID(HealPointsEnduranceConfigManualSelectSpell)];
		updateRankLists();
		HEALPOINTS_CONFIG['endurance']['rank'] = 1;
		UIDropDownMenu_SetSelectedID(HealPointsEnduranceConfigManualSelectRank, 1);		
		HealPointsCalculator_ConfigUpdated();
	end
end

function HealPointsConfigRankSelect_OnClick()
	if (this.owner) then
		UIDropDownMenu_SetSelectedID(this.owner, this:GetID());
		HEALPOINTS_CONFIG['power']['rank'] = UIDropDownMenu_GetSelectedID(HealPointsPowerConfigManualSelectRank);
		HEALPOINTS_CONFIG['endurance']['rank'] = UIDropDownMenu_GetSelectedID(HealPointsEnduranceConfigManualSelectRank);
		HealPointsCalculator_ConfigUpdated();
	end
end

function HealPointsConfig_OnLoad()
	tinsert(UISpecialFrames, "HealPointsConfig"); -- Esc closes the window

	if (HEALPOINTS_CONFIG['power'] == nil) then
		HEALPOINTS_CONFIG['power'] = { auto = 1, duration = '1', spell = '', rank = 1 };
	end
	if (HEALPOINTS_CONFIG['endurance'] == nil) then
		HEALPOINTS_CONFIG['endurance'] = { auto = 1, duration = '5', spell = '', rank = 1};
	end
	if (HEALPOINTS_CONFIG['regen'] == nil) then
		HEALPOINTS_CONFIG['regen'] = { duration = '3' };
	end

end

function HealPointsConfig_Update()
	if (valueUpdating == 0) then
		-- PowerPoints
		HEALPOINTS_CONFIG['power']['duration'] = getglobal("HealPointsPowerConfigDuration"):GetValue();
		HEALPOINTS_CONFIG['power']['auto'] = getglobal("HealPointsPowerConfigAutoSelect"):GetChecked();
		
		-- EndurancePoints
		HEALPOINTS_CONFIG['endurance']['duration'] = getglobal("HealPointsEnduranceConfigDuration"):GetValue();
		HEALPOINTS_CONFIG['endurance']['auto'] = getglobal("HealPointsEnduranceConfigAutoSelect"):GetChecked();
	
		-- RegenPoints
		HEALPOINTS_CONFIG['regen']['duration'] = getglobal("HealPointsRegenConfigDuration"):GetValue();
	
		getglobal("HealPointsPowerConfigDurationText"):SetText(HEALPOINTS_CONFIG['power']['duration'].." min");
		getglobal("HealPointsEnduranceConfigDurationText"):SetText(HEALPOINTS_CONFIG['endurance']['duration'].." min");
		getglobal("HealPointsRegenConfigDurationText"):SetText(HEALPOINTS_CONFIG['regen']['duration'].." min");
		
		HealPointsCalculator_ConfigUpdated();
	end
end

function HealPointsConfigReset_OnClick()
	HEALPOINTS_CONFIG['power'] = { auto = 1, duration = '1', spell = SPELLLIST[1], rank = 1 };
	HEALPOINTS_CONFIG['endurance'] = { auto = 1, duration = '5', spell = SPELLLIST[1], rank = 1};
	HEALPOINTS_CONFIG['regen'] = { duration = '3' };
	updateGUI();
	
	HealPointsCalculator_ConfigUpdated();	
end

function HealPointsConfig_OnShow()
	if (table.getn(SPELLLIST) == 0) then
		updateSpellLists();
	end
	updateGUI();
end

function HealPointsConfig_OnMouseDown(arg1)
	if (arg1=="LeftButton") then
		HealPointsConfig:StartMoving();
	end
end

function HealPointsConfig_OnMouseUp(arg1)
	if (arg1=="LeftButton") then
		HealPointsConfig:StopMovingOrSizing();
	end
end
